import {Component} from '@angular/core';
import {FormBuilder, FormGroup, ReactiveFormsModule, Validators} from "@angular/forms";
import {NavigationService} from "@service/navigation.service";
import SetupService from "@api/v1/setup.service";
import {DateAdapter, MatNativeDateModule} from "@angular/material/core";
import {MatFormFieldModule} from "@angular/material/form-field";
import {MatSelectModule} from "@angular/material/select";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {MatIconButton} from "@angular/material/button";
import {MatIcon} from "@angular/material/icon";
import {MatDatepickerModule} from "@angular/material/datepicker";

@Component({
  selector: 'app-schedule-conf',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    MatFormFieldModule,
    MatSelectModule,
    MatInput,
    MatTooltip,
    MatIconButton,
    MatIcon,
    MatDatepickerModule,
    MatNativeDateModule
  ],
  templateUrl: './schedule.component.html',
})

export class ScheduleComponent {
  protected scheduleSettings!: FormGroup;

  constructor(
    private navigationService: NavigationService,
    formBuilder: FormBuilder,
    private api: SetupService,
    adapter: DateAdapter<any>) {
    adapter.setLocale(navigator.language);
    this.scheduleSettings = formBuilder.group({
        cron: ['0 */6 * * *', Validators.pattern(/^(\S+\s){4}\S$/)],
        startTerm: ['', Validators.required]
      }
    );

    this.navigationService.setNextButtonState(false);
    this.scheduleSettings.valueChanges.subscribe(() => {
      this.navigationService.setNextButtonState(this.scheduleSettings.valid);
    });

    this.navigationService.nextButtonAction = () => {
      return this.api.setSchedule({
          "cronUpdateSchedule": this.scheduleSettings.get('cron')?.value,
          "startTerm": this.scheduleSettings.get('startTerm')?.value
        }
      );
    };
  }
}
